from .render_categorical import render_categorical
from .render_date import render_date
from . render_numerical import render_numerical
from .render_generic import render_generic
from .render_string import render_string


from dataclasses import dataclass
from typing import Any, Dict, Optional, List

import pandas as pd

@dataclass
class TableDescription:
    df: pd.DataFrame = None
    dataset_statistics: Dict = None
    correlation: pd.DataFrame = None
    alerts: List = None
    variable_types: Dict = None
    variables: Optional[Dict] = None
    comparison: Dict = None
    shared_values: List[int] = None

@dataclass
class ComparisonDescription:
    df: List[pd.DataFrame]
    value_count: List[int]
    distinct_count: List[int]
    shared_values: List[int]


import pandas as pd

from xurpas_data_quality.render.renderer import HTMLBase, HTMLContainer
from xurpas_data_quality.render.render_empty import render_empty

def get_empty_report(df: pd.DataFrame, name:str)-> HTMLBase:
    """
    Generates an empty report
    """

    body = HTMLContainer(type="sections",
                         container_items = render_empty(df, name))


    return HTMLBase(body=body, name='Data Report' if name is None else name)


import pandas as pd

from xurpas_data_quality.data.descriptions import TableDescription
from xurpas_data_quality.render.renderer import BaseRenderer, HTMLBase,HTMLContainer
from xurpas_data_quality.render.render_error import render_error
from xurpas_data_quality.render.render import render_report
from xurpas_data_quality.render.render_empty import render_empty

def get_error_report(data:TableDescription|pd.DataFrame, invalid_data:TableDescription, errors:list,name:str, is_empty:bool, minimal:bool)-> BaseRenderer:
    """
    Generates an error report
    """

    content = [render_error(data, invalid_data, errors, is_empty=is_empty)]
    if is_empty:
        content.extend(render_empty(data, name))
    else:
        content.extend(render_report(data=data, report_name=name, minimal=minimal))
    
    body = HTMLContainer(
        type="sections",
        container_items = content
    )


    return HTMLBase(body=body, name='Data Report' if name is None else name)


import pandas as pd

from typing import List

from xurpas_data_quality.data.descriptions import TableDescription
from xurpas_data_quality.render.renderer import HTMLBase, HTMLContainer, HTMLTable
from xurpas_data_quality.config import Settings

def render_compare(data: List[TableDescription], table_names:List[str],name:str):
    
    config = {'color_list': Settings().colors.base_colors}

    headers = [f"Table {i+1}" for i in range(len(data))] if table_names is None else table_names

    distinct_count_list = []
    distinct_perc_list = []
    value_count_list = []
    shared_values_list = []
    shared_values_count_list = []
    shared_values_perc_list = []
    variable_types_list = []
    num_variables_list = []
    missing_cells_list = []
    missing_cells_perc_list = []
    duplicate_rows_list = []
    duplicate_rows_perc_list = []
    unshared_values_list = []
    dataframe_comparison = []

    for datum in data:
        distinct_count_list.append(datum.comparison['distinct_count'])
        value_count_list.append(datum.comparison['value_count'])
        distinct_perc_list.append(datum.comparison['distinct_perc'])
        shared_values_list.append(datum.shared_values['shared_values'])
        shared_values_count_list.append(datum.shared_values['shared_values_count'])
        shared_values_perc_list.append((datum.shared_values['shared_values_count']/datum.comparison['value_count'])*100)
        unshared_values_list.append((datum.comparison['unshared_values']))

        variable_types_list.append(datum.variable_types)
        num_variables_list.append(datum.dataset_statistics['num_variables'])
        missing_cells_list.append(datum.dataset_statistics['missing_cells'])
        missing_cells_perc_list.append("{:0.2f}%".format(datum.dataset_statistics['missing_cells_perc']))
        duplicate_rows_list.append(datum.dataset_statistics['duplicate_rows'])
        duplicate_rows_perc_list.append("{:0.2f}%".format(datum.dataset_statistics['duplicate_rows_perc']))

        dataframe_comparison.append(datum.comparison['unshared_df'])
    
    combined_data = {
        'Count of Values': value_count_list,
        'Distinct Values (Count)': distinct_count_list,
        'Distinct Values (Percentage)': [f"{x:.2f}%" for x in distinct_perc_list],
        'Values Existing in All Tables (Count)': shared_values_count_list,
        'Values Existing in All Tables (Percentage)': [f"{x:.2f}%" for x in shared_values_perc_list],
        'Values NOT Shared between the tables': unshared_values_list
    }  

    dataset_statistics = {
        'Number of Variables': num_variables_list,
        'Missing Cells': missing_cells_list,
        'Missing Cells (%)': missing_cells_perc_list,
        'Duplicate Rows': duplicate_rows_list,
        'Duplicate Rows (%)': duplicate_rows_perc_list 
    }

    variable_types_statistics = {}
    for key in set().union(*variable_types_list):
        variable_types_statistics[key] = [0] * len(variable_types_list)

    for i, d in enumerate(variable_types_list):
        for key, value in d.items():
            variable_types_statistics[key][i] = value

    dataframe_unshared_sample = []

    for i, dataframe in enumerate(dataframe_comparison):
        dataframe_unshared_sample.append(
            HTMLContainer(
                type="default",
                name = headers[i],
                id = f"unshared_{headers[i]}_container",
                container_items=[
                    HTMLTable(
                    id = f"unshared_{headers[i]}",
                    name = f"Rows in {headers[i]} that are not shared with the other tables",
                    data= dataframe.to_html(classes="table table-sm", border=0, index=True, justify='left')
                    )
                ]
            ) 
        )    

    unshared_samples_tabs = HTMLContainer(
        type="tabs",
        container_items= dataframe_unshared_sample
    )

    comparison_section = HTMLContainer(
        type="tabs",
        container_items=[
            HTMLContainer(
                type="default",
                name="Comparison",
                id="comparison-column-compare",
                container_items=[HTMLTable(
                    id = "comparison",
                    name = '"Volume Column" Comparison',
                    data = combined_data,
                    headers = headers,
                    config = config
                )]
            ),
            HTMLContainer(
                type="default",
                name="Samples",
                id="samples-column-compare",
                container_items=[
                    unshared_samples_tabs
                ]
            )
        ]
    )

    column_comparison = HTMLContainer(
            type="box",
            name = "Comparison",
            container_items=[
                comparison_section
            ]
        )
    
    overview_section = HTMLContainer(
        type="box",
        name="Overview",
        container_items = [
            HTMLContainer(
                type="tabs",
                container_items=[HTMLContainer(
                    type="default",
                    name="Data",
                    id="comparison-data",
                    container_items = [HTMLTable(
                        id = "comparison",
                        name = "Overview of Data",
                        data = dataset_statistics,
                        headers = headers,
                        config = config
                        )]
                    ),
                    HTMLContainer(
                        type="default",
                        name="Variables",
                        id = "comparison-variables",
                        container_items = [HTMLTable(
                            id = "comparison",
                            name = "Overview of Variables",
                            data = variable_types_statistics,
                            headers = headers,
                            config = config
                        )]
                    )
                ]
            ),
        ]
    )

    content = [
        column_comparison,
        overview_section
    ]
    body = HTMLContainer(type="sections",
                         container_items = content)

    if name is not None:
        return HTMLBase(
            body=body,
            name=name
        )
    
    else:
        return HTMLBase(
            body=body
        )


import pandas as pd
from xurpas_data_quality.render.renderer import HTMLBase, HTMLContainer
from xurpas_data_quality.render.render_sections import render_variables_section, render_dropdown_section


def render_test(data):
    variables_section = HTMLContainer(
        type = "box",
        name = "Variables",
        container_items = render_dropdown_section(items=render_variables_section(data), names=list(data.df))
    )

    body = HTMLContainer(type="sections",
                        container_items = [variables_section])
    
    return HTMLBase(
        body=body,
        name="report.py"
    )


from .describer import Describer
from .typeset import XTypeSet, Numeric, Text, Categorical
from .series_describe import get_series_descriptions
from .correlations import get_correlations
from .describe import describe, describe_invalid, check_data
from .dataframe import load_dataframe, validate_dataframe, convert_to_pandas
from .utils import check_dtypes, check_col_names
from .descriptions import TableDescription, ComparisonDescription


from typing import Any, List

from xurpas_data_quality.data.descriptions import TableDescription
from xurpas_data_quality.render.renderer import BaseRenderer, HTMLBase, HTMLContainer, HTMLTable, HTMLVariable, HTMLPlot, HTMLToggle, HTMLCollapse, HTMLDropdown
from xurpas_data_quality.visuals import plot_to_base64, create_missing_bar_plot
from xurpas_data_quality.render.render_sections import render_variables_section, render_correlation_section, render_interactions_section, render_dropdown_section


def render_report(data: TableDescription, minimal:bool,report_name: str=None) -> List[BaseRenderer]:
    """
    Creates the content of a normal report
    """
    
    content = []
    dataset_statistics = {
        'Number of Variables': data.dataset_statistics['num_variables'],
        'Missing Cells': data.dataset_statistics['missing_cells'],
        'Missing Cells (%)': "{:0.2f}%".format(data.dataset_statistics['missing_cells_perc']),
        'Duplicate Rows': data.dataset_statistics['duplicate_rows'],
        'Duplicate Rows (%)': "{:0.2f}%".format(data.dataset_statistics['duplicate_rows_perc'])
    }

    if data.alerts is not None and "shipment_no_alerts" in data.alerts:
        overview= HTMLContainer(
            type="default",
            name="Overview",
            id="overview-tab",
            container_items = [
                HTMLContainer(
                    type="column",
                    container_items = HTMLTable(
                        data = dataset_statistics,
                        name="Dataset Statistics"
                    )),
                HTMLContainer(
                    type="column",
                    container_items =  HTMLTable(
                        data= data.variable_types,
                        name="Variable Types"
                    )
                )
            ]
        )
        alerts = HTMLContainer(
            type="default",
            name="Alerts",
            id="alerts-tab",
            flex=True,
            container_items = [
                HTMLTable(
                    id="alerts",
                    data=[f" There are {data.alerts['shipment_no_alerts']['alerts']} rows with no Shipment Numbers"]
                ),
                HTMLTable(
                    id ="alert_sample",
                    data = data.alerts["shipment_no_alerts"]["df"].to_html(classes="table table-sm", border=0, justify='left')
                )
            ])
        
        overview_section = HTMLContainer(
            type="box",
            name="Overview",
            id = "overview_section",
            container_items=[
                HTMLContainer(
                    type="tabs",
                    container_items=[overview, alerts]
                )
            ]
        )

    else:
        overview_section = HTMLContainer(
            type="box",
            name="Overview",
            container_items = [
                HTMLContainer(
                    type="column",
                    container_items = HTMLTable(
                        data = dataset_statistics,
                        name="Dataset Statistics"
                    )),
                HTMLContainer(
                    type="column",
                    container_items =  HTMLTable(
                        data= data.variable_types,
                        name="Variable Types"
                    )
                )
            ]
        )

    variables_section = HTMLContainer(
        type = "box",
        name = "Variables",
        container_items = render_dropdown_section(items=render_variables_section(data), names=list(data.df))
    )

    missing_section = HTMLContainer(
        type="box",
        name="Missing",
        container_items=[
            HTMLPlot(plot=plot_to_base64(create_missing_bar_plot(data.df), minimal=minimal),
            type="large",
            id="missingplot",
            name="Missing Bar Plot")]
    )

    correlation_section = render_correlation_section(data.correlation, minimal)


    samples_section = HTMLContainer(
        type="box",
        name="Sample",
        container_items=[
            HTMLTable(
                id = "sample",
                data=data.df.head(10).to_html(classes="table table-sm", border=0, index=False, justify='left')
            )
        ]
    )
    
    content.extend([overview_section,
                samples_section,
                variables_section,
                missing_section, 
                correlation_section, 
                ])

    if not minimal:
        interactions_section = HTMLContainer(
        type="box",
        name="Interactions",
        container_items=[
            HTMLContainer(
                type="tabs",
                container_items= render_interactions_section(data.df, minimal)
                )
            ]
        )

        content.extend([interactions_section])

    return content


from typing import List

from xurpas_data_quality.data.descriptions import TableDescription
from xurpas_data_quality.render.renderer import HTMLBase
from xurpas_data_quality.render.render_compare import render_compare

def get_comparison_report(data:List[TableDescription], table_names: List[str],name:str)-> HTMLBase:
    """
    Generates a comparison report
    """

    return render_compare(data, table_names,name)